#
# Copyright (c) 2002 Gilles Lenfant
# All rights reserved. Written by Gilles Lenfant <glenfant@bigfoot.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
'''
This module provides resources for building a Quizz object from XML
The DTD for this XML is the file "quizz.dtd"

This module exposes the parseXQuizz function

$Id: xquizzparser.py,v 1.1.1.1 2002/05/27 02:22:35 glt Exp $
'''

__version__ = "$Revision: 1.1.1.1 $"[11:-2]

__all__ = ['parseXQuizz']

import sys
import string
import types
from exceptions import Exception

from xml.unicode.iso8859 import wstring
from xml.dom.minidom import parseString

from quizz import Quizz, Question, Answer

##########################################################
# Fixing encoding depending on Python and PyXML versions #
##########################################################

def encode_asis(text, encoding):
    # Buggy old PyXML (v0.6.6 -)
    return text

def encode_py2(text, encoding):
    # Python 2.x native Unicode features
    if type(text) == types.UnicodeType:
        return text.encode(encoding)
    else:
        return text

def encode_wstring(text, encoding):
    return wstring.decode("UTF-8",text).encode(encoding)

if sys.version[0] == '1':
    # Python 1.5.2
    if xml.version_info >= (0, 7, 0):
        recode = encode_wstring
    else:
        recode = encode_asis
else:
    # Python 2.x
    recode = encode_py2

#####################################
# Parse xml and make a Quizz object #
#####################################

class XQuizzContentError(Exception):
    def __init__(self, value):
        self.value=value
        return
    def __str__(self):
        return str(self.value)

def _textOfNodes(nodelist, encoding):
    rc = ''
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc = rc + node.data
    return string.strip(recode(rc, encoding))

def parseXQuizz(xmlQuizz, encoding='ISO-8859-1'):
    '''makes a Quizz object from an XML string (xmlQuizz)
    encoding is the prefered (8 bit) encoding to use
    -> (errors, quizz)
    errors is a list of error strings
    quizz is the generated Quizz object (None if errors != [])
    '''
    errors = []
    quizz = Quizz()
    try:
        dt = parseString(xmlQuizz)
        dt.normalize()
        if dt.documentElement.tagName != 'quizz':
            errors.append('root element should be "<quizz>"')
        xquestions = dt.getElementsByTagName('question')
        if len(xquestions) < 1:
            raise XQuizzContentError, '"<quizz>" should have one or more "<question>" elements'
        iquestion = 0
        for xquestion in xquestions:
            iquestion = iquestion + 1
            question = Question()
            xlabels = xquestion.getElementsByTagName('label')
            if len(xlabels) != 1:
                raise XQuizzContentError, '"<question>" #%d should have one "<label>" element' % iquestion
            label = _textOfNodes(xlabels[0].childNodes, encoding)
            question.label = label
            xanswers = xquestion.getElementsByTagName('answer')
            if len(xanswers) < 2:
                raise XQuizzContentError, '"<question>" #%d should have two or more "<answer>" elements' % iquestion
            ianswer = 0
            for xanswer in xanswers:
                ianswer = ianswer + 1
                answer = Answer()
                try:
                    value = int(xanswer.getAttribute('value'))
                except:
                    raise XQuizzContentError, ('"<answer>" #%d of "<question>" #%s should have an integer attribute named "value"'
                                               % (ianswer, iquestion))
                answer.points = value
                label = _textOfNodes(xanswer.childNodes, encoding)
                answer.label = label
                question.addAnswer(answer)
            # /for...
            quizz.addQuestion(question)
        # /for...
    except XQuizzContentError, e:
        errors.append(str(e))
    except:
        # Misc XML parsing errors
        errors.append(str(sys.exc_info()[1]))
    try:
        dt.unlink()
    except:
        # Doesn't matter
        pass
    if errors != []:
        return errors, None
    else:
        return [], quizz
# /def parseXQuizz
